from ._slackrequest import SlackRequest
from ._channel import Channel
from ._user import User
from ._util import SearchList

from websocket import create_connection
import json


class Server(object):
    def __init__(self, token, connect=True):
        self.token = token
        self.username = None
        self.domain = None
        self.login_data = None
        self.websocket = None
        self.users = SearchList()
        self.channels = SearchList()
        self.connected = False
        self.pingcounter = 0
        self.api_requester = SlackRequest()

        if connect:
            self.rtm_connect()

    def __eq__(self, compare_str):
        if compare_str == self.domain or compare_str == self.token:
            return True
        else:
            return False

    def __str__(self):
        data = ""
        for key in list(self.__dict__.keys()):
            data += "{} : {}\n".format(key, str(self.__dict__[key])[:40])
        return data

    def __repr__(self):
        return self.__str__()

    def rtm_connect(self, reconnect=False):
        reply = self.api_requester.do(self.token, "rtm.start")
        if reply.code != 200:
            raise SlackConnectionError
        else:
            login_data = json.loads(reply.read().decode('utf-8'))
            import ipdb; ipdb.set_trace
            if login_data["ok"]:
                self.ws_url = login_data['url']
                if not reconnect:
                    self.parse_slack_login_data(login_data)
                self.connect_slack_websocket(self.ws_url)
            else:
                raise SlackLoginError

    def parse_slack_login_data(self, login_data):
        self.login_data = login_data
        self.domain = self.login_data["team"]["domain"]
        self.username = self.login_data["self"]["name"]
        self.parse_channel_data(login_data["channels"])
        self.parse_channel_data(login_data["groups"])
        self.parse_channel_data(login_data["ims"])
        self.parse_user_data(login_data["users"])

    def connect_slack_websocket(self, ws_url):
        try:
            self.websocket = create_connection(ws_url)
            self.websocket.sock.setblocking(0)
        except:
            raise SlackConnectionError

    def parse_channel_data(self, channel_data):
        for channel in channel_data:
            if "name" not in channel:
                channel["name"] = channel["id"]
            if "members" not in channel:
                channel["members"] = []
            self.attach_channel(channel["name"],
                                channel["id"],
                                channel["members"])

    def parse_user_data(self, user_data):
        for user in user_data:
            if "tz" not in user:
                user["tz"] = "unknown"
            if "real_name" not in user:
                user["real_name"] = user["name"]
            self.attach_user(user["name"], user["id"], user["real_name"], user["tz"])

    def send_to_websocket(self, data):
        """Send (data) directly to the websocket."""
        try:
            data = json.dumps(data)
            self.websocket.send(data)
        except:
            self.rtm_connect(reconnect=True)

    def ping(self):
        return self.send_to_websocket({"type": "ping"})

    def websocket_safe_read(self):
        """ Returns data if available, otherwise ''. Newlines indicate multiple
            messages
        """

        data = ""
        while True:
            try:
                data += "{}\n".format(self.websocket.recv())
            except:
                return data.rstrip()

    def attach_user(self, name, id, real_name, tz):
        self.users.append(User(self, name, id, real_name, tz))

    def attach_channel(self, name, id, members=[]):
        self.channels.append(Channel(self, name, id, members))

    def join_channel(self, name):
        print(self.api_requester.do(self.token,
                                    "channels.join?name={}".format(name)).read())

    def api_call(self, method, **kwargs):
        reply = self.api_requester.do(self.token, method, kwargs)
        return reply.read()


class SlackConnectionError(Exception):
    pass


class SlackLoginError(Exception):
    pass



#!/usr/bin/python
# mostly a proxy object to abstract how some of this works

import json

from ._server import Server

class SlackClient(object):
    def __init__(self, token):
        self.token = token
        self.server = Server(self.token, False)

    def rtm_connect(self):
        try:
            self.server.rtm_connect()
            return True
        except:
            return False

    def api_call(self, method, **kwargs):
        return self.server.api_call(method, **kwargs)

    def rtm_read(self):
        # in the future, this should handle some events internally i.e. channel
        # creation
        if self.server:
            json_data = self.server.websocket_safe_read()
            data = []
            if json_data != '':
                for d in json_data.split('\n'):
                    data.append(json.loads(d))
            for item in data:
                self.process_changes(item)
            return data
        else:
            raise SlackNotConnected

    def rtm_send_message(self, channel, message):
        return self.server.channels.find(channel).send_message(message)

    def process_changes(self, data):
        if "type" in data.keys():
            if data["type"] == 'channel_created':
                channel = data["channel"]
                self.server.attach_channel(channel["name"], channel["id"], [])
            if data["type"] == 'im_created':
                channel = data["channel"]
                self.server.attach_channel(channel["user"], channel["id"], [])
            pass


class SlackNotConnected(Exception):
    pass



from .slackclient._util import SearchList
from .slackclient._user import User

class FakeServer(object):
    def __init__(self, slack=None, config=None, hooks=None, db=None):
        self.slack = slack or FakeSlack()
        self.config = config
        self.hooks = hooks
        self.db = db

    def query(self, sql, *params):
        # XXX: what to do with this?
        return None

class FakeSlack(object):
    def __init__(self, server=None, users=None):
        self.server = server or FakeSlackServer(users=users)

class FakeSlackServer(object):
    def __init__(self, botname="limbo_test", users=None):
        self.login_data = {
            "self": {
                "name": botname,
            }
        }

        if  users:
            self.users = users
        else:
            self.users = SearchList([
                User(self, "limbo_test", 1, "", 0),
                User(self, "msguser", 2, "", 0),
                User(self, "slackbot", 3, "", 0),
            ])



class User(object):
    def __init__(self, server, name, id, real_name, tz):
        self.tz = tz
        self.name = name
        self.real_name = real_name
        self.server = server
        self.id = id

    def __eq__(self, compare_str):
        if self.id == compare_str or self.name == compare_str:
            return True
        else:
            return False

    def __str__(self):
        data = ""
        for key in list(self.__dict__.keys()):
            if key != "server":
                data += "{} : {}\n".format(key, str(self.__dict__[key])[:40])
        return data

    def __repr__(self):
        return self.__str__()



# -*- coding: utf-8 -*-
"""!mlb <team> will return that team's upcoming games"""

from datetime import datetime
import re

import requests
from bs4 import BeautifulSoup as Soup

schedules = {
    'arizona diamondbacks': 'http://espn.go.com/mlb/team/schedule/_/name/ari/arizona-diamondbacks',
    'atlanta braves': 'http://espn.go.com/mlb/team/schedule/_/name/atl/atlanta-braves',
    'baltimore orioles': 'http://espn.go.com/mlb/team/schedule/_/name/bal/baltimore-orioles',
    'boston red sox': 'http://espn.go.com/mlb/team/schedule/_/name/bos/boston-red-sox',
    'chicago cubs': 'http://espn.go.com/mlb/team/schedule/_/name/chc/chicago-cubs',
    'chicago white sox': 'http://espn.go.com/mlb/team/schedule/_/name/chw/chicago-white-sox',
    'cincinnati reds': 'http://espn.go.com/mlb/team/schedule/_/name/cin/cincinnati-reds',
    'cleveland indians': 'http://espn.go.com/mlb/team/schedule/_/name/cle/cleveland-indians',
    'colorado rockies': 'http://espn.go.com/mlb/team/schedule/_/name/col/colorado-rockies',
    'detroit tigers': 'http://espn.go.com/mlb/team/schedule/_/name/det/detroit-tigers',
    'houston astros': 'http://espn.go.com/mlb/team/schedule/_/name/hou/houston-astros',
    'kansas city royals': 'http://espn.go.com/mlb/team/schedule/_/name/kc/kansas-city-royals',
    'los angeles angels': 'http://espn.go.com/mlb/team/schedule/_/name/laa/los-angeles-angels',
    'los angeles dodgers': 'http://espn.go.com/mlb/team/schedule/_/name/lad/los-angeles-dodgers',
    'miami marlins': 'http://espn.go.com/mlb/team/schedule/_/name/mia/miami-marlins',
    'milwaukee brewers': 'http://espn.go.com/mlb/team/schedule/_/name/mil/milwaukee-brewers',
    'minnesota twins': 'http://espn.go.com/mlb/team/schedule/_/name/min/minnesota-twins',
    'new york mets': 'http://espn.go.com/mlb/team/schedule/_/name/nym/new-york-mets',
    'new york yankees': 'http://espn.go.com/mlb/team/schedule/_/name/nyy/new-york-yankees',
    'oakland athletics': 'http://espn.go.com/mlb/team/schedule/_/name/oak/oakland-athletics',
    'philadelphia phillies': 'http://espn.go.com/mlb/team/schedule/_/name/phi/philadelphia-phillies',
    'pittsburgh pirates': 'http://espn.go.com/mlb/team/schedule/_/name/pit/pittsburgh-pirates',
    'san diego padres': 'http://espn.go.com/mlb/team/schedule/_/name/sd/san-diego-padres',
    'san francisco giants': 'http://espn.go.com/mlb/team/schedule/_/name/sf/san-francisco-giants',
    'seattle mariners': 'http://espn.go.com/mlb/team/schedule/_/name/sea/seattle-mariners',
    'st. louis cardinals': 'http://espn.go.com/mlb/team/schedule/_/name/stl/st-louis-cardinals',
    'tampa bay rays': 'http://espn.go.com/mlb/team/schedule/_/name/tb/tampa-bay-rays',
    'texas rangers': 'http://espn.go.com/mlb/team/schedule/_/name/tex/texas-rangers',
    'toronto blue jays': 'http://espn.go.com/mlb/team/schedule/_/name/tor/toronto-blue-jays',
    'washington nationals': 'http://espn.go.com/mlb/team/schedule/_/name/wsh/washington-nationals'
}

def fmtdatetime(dt):
    hour = datetime.strftime(dt, "%I").lstrip('0')
    return datetime.strftime(dt, "%m/%d {0}%p".format(hour))

def schedule(query):
    url = None
    query = query.lower()
    for team in schedules:
        if query in team:
            url = schedules[team]
            break

    if not url:
        return "Unable to find {0}".format(query)

    r = requests.get(url)
    soup = Soup(r.text)
    sched = soup.find("table", attrs={"class": "tablehead"})
    games = []
    for row in sched.findAll("tr")[2:]:
        # month name rows have OPPONENT in them
        if "OPPONENT" in row.text:
            continue

        rawdt, rawopp, time = [t.text for t in row.findAll("td")][0:3]
        yr = datetime.strftime(datetime.now(), "%Y")
        try:
            dt = datetime.strptime("{0} {1} {2}".format(rawdt, yr, time), "%a, %b %d %Y %I:%M %p")
        except ValueError:
            # some games are TBA
            dt = datetime.strptime(dt, "%a, %b %d")
        # away games come as @, which is fine. Home games should have the
        # leading "vs" stripped
        opp = rawopp.lstrip("vs")
        games.append((dt, opp))

    next3 = [
        "{0} {1}".format(fmtdatetime(dt), opp)
        for dt, opp
        in games
        if dt > datetime.now()
    ][:3]
    return "{0}: ".format(team.title()) + " :baseball: ".join(next3)

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!mlb (.*)", text)
    if not match:
        return

    return schedule(match[0])



import time

try:
    # Try for Python3
    from urllib.parse import urlencode 
    from urllib.request import urlopen 
except:
    # Looks like Python2
    from urllib import urlencode 
    from urllib2 import urlopen 


class SlackRequest(object):
    def __init__(self):
        pass

    def do(self, token, request="?", post_data={}, domain="slack.com"):
        post_data["token"] = token
        post_data = urlencode(post_data)
        url = 'https://{}/api/{}'.format(domain, request)
        return urlopen(url, post_data.encode('utf-8'))

